/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2020 IndiScale GmbH <info@indiscale.com>
 * Copyright (C) 2020 Timm Fitschen <t.fitschen@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */

package org.caosdb.server.jobs;

/**
 * ScheduledJob is a wrapper class for jobs held by the Scheduler.
 *
 * <p>It is mainly a means to have simplified interface for the Scheduler which also measures the
 * execution time of the job "from outside".
 *
 * @author Timm Fitschen (t.fitschen@indiscale.com)
 */
public class ScheduledJob {

  long runtime = 0;
  final Job job;
  private long startTime = -1;

  ScheduledJob(final Job j) {
    this.job = j;
  }

  void run() {
    if (!hasStarted()) {
      start();
      this.job.run();
      finish();
    }
  }

  @Override
  public String toString() {
    return this.job.toString();
  }

  private void start() {
    this.startTime = System.currentTimeMillis();
  }

  private void finish() {
    this.runtime += System.currentTimeMillis() - this.startTime;
    this.job
        .getContainer()
        .getTransactionBenchmark()
        .addMeasurement(this.job.getClass().getSimpleName(), this.runtime);
  }

  void pause() {
    this.runtime += System.currentTimeMillis() - this.startTime;
  }

  void unpause() {
    start();
  }

  private boolean hasStarted() {
    return this.startTime != -1;
  }

  public JobExecutionTime getExecutionTime() {
    return this.job.getExecutionTime();
  }

  public boolean skip() {
    return this.job.getTarget().skipJob();
  }
}
